/*****************************************************************************
 *
 * openastro.h -- Main Open Astro Project API header
 *
 * Copyright 2014 James Fidell (james@openastroproject.org)
 *
 * License:
 *
 * This file is part of the Open Astro Project.
 *
 * The Open Astro Project is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The Open Astro Project is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Open Astro Project.  If not, see
 * <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/

#ifndef OPENASTRO_H
#define OPENASTRO_H

#if HAVE_LIMITS_H
#include <limits.h>
#endif

#include <openastro/errno.h>
#include <openastro/debug.h>
#include <openastro/controlTypes.h>

typedef struct {
  const int		interfaceType;
  const char*		name;
  const char*		shortName;
  int			(*enumerate)();
  const unsigned int	userConfigFlags;
} oaInterface;

typedef struct {
  unsigned long         devIndex;
  unsigned long         devType;
  unsigned short        vendorId;
  unsigned short        productId;
  unsigned long         misc;
  uint64_t              guid;
  uint8_t               unit;
  unsigned int          colour;
  char                  sysPath[ PATH_MAX+1 ];
} DEVICE_INFO;

#define	OA_DEVICE_CAMERA	1
#define	OA_DEVICE_FILTERWHEEL	2

#endif	/* OPENASTRO_H */
