/*****************************************************************************
 *
 * atikSerialoacam.h -- header for Atik serial (FTDI-based) camera API
 *
 * Copyright 2013,2014,2015 James Fidell (james@openastroproject.org)
 *
 * License:
 *
 * This file is part of the Open Astro Project.
 *
 * The Open Astro Project is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The Open Astro Project is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Open Astro Project.  If not, see
 * <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/

#ifndef OA_ATIK_SERIAL_OACAM_H
#define OA_ATIK_SERIAL_OACAM_H

#ifndef HAVE_LIBUDEV
#ifdef HAVE_LIBFTDI1_FTDI_H
#include <libftdi1/ftdi.h>
#endif
#ifdef HAVE_LIBFTDI_FTDI_H
#include <libftdi/ftdi.h>
#endif
#endif

extern int		oaAtikSerialGetCameras ( CAMERA_LIST* );
extern oaCamera*	oaAtikSerialInitCamera ( oaCameraDevice* );
extern int              oaAtikSerialCloseCamera ( oaCamera* );

extern int		oaAtikSerialCameraTestControl ( oaCamera*, int,
			    oaControlValue* );
extern int		oaAtikSerialCameraSetControl ( oaCamera*, int,
			    oaControlValue* );
extern int		oaAtikSerialCameraReadControl ( oaCamera*, int,
			    oaControlValue* );
extern int		oaAtikSerialCameraGetControlRange ( oaCamera*, int,
			    int64_t*, int64_t*, int64_t*, int64_t* );

extern int		oaAtikSerialCameraStartStreaming ( oaCamera*,
				void* (*)(void*, void*, int), void* );
extern int		oaAtikSerialCameraStopStreaming ( oaCamera* );
extern int		oaAtikSerialCameraIsStreaming ( oaCamera* );

extern int		oaAtikSerialCameraSetResolution ( oaCamera*, int, int );

extern void*		oaAtikSerialcontroller ( void* );
extern void*		oaAtikSerialcallbackHandler ( void* );

extern const FRAMESIZES* oaAtikSerialCameraGetFrameSizes ( oaCamera* );
extern int		oaAtikSerialCameraGetFramePixelFormat ( oaCamera*,int);

#ifdef HAVE_LIBUDEV
extern int              _atikSerialCamWrite ( int, const unsigned char*, int );
extern int              _atikSerialCamRead ( int, unsigned char*, int );
extern int              _atikSerialCamReadToZero ( int, unsigned char*,
				unsigned int );
extern int		_atikSerialCamReadBlock ( int, unsigned char*, int );
#else
extern int              _atikSerialCamWrite ( struct ftdi_context*,
                            const unsigned char*, int );
extern int              _atikSerialCamRead ( struct ftdi_context*,
                            unsigned char*, int );
extern int              _atikSerialCamReadToZero ( struct ftdi_context*,
                            unsigned char*, unsigned int );
extern int		_atikSerialCamReadBlock ( struct ftdi_context*,
                            unsigned char*, int );
#endif

struct atikSerialCam {
  unsigned int	vendorId;
  unsigned int	productId;
  const char*	name;
  unsigned int	devType;
};

#define	NULL_READS		20
#define OPTIMAL_READ_SIZE	3968

#define DEFAULT_EXPOSURE        500

#endif	/* OA_ATIK_SERIAL_OACAM_H */
