/*****************************************************************************
 *
 * sxIO.c -- Starlight Xpress filter wheel IO routines
 *
 * Copyright 2014 James Fidell (james@openastroproject.org)
 *
 * License:
 *
 * This file is part of the Open Astro Project.
 *
 * The Open Astro Project is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The Open Astro Project is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Open Astro Project.  If not, see
 * <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/

#include <oa_common.h>

#include <libusb-1.0/libusb.h>

#include <errno.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>
#include <openastro/filterwheel.h>

#include "sxfw.h"

#define	wheelState		wheel->_sx


int
_sxWheelWrite ( oaFilterWheel* wheel, char* buffer, int len )
{
  int lenOffset = 0;
  int xferred = 0;
  int reportNum = buffer[0];
  int res;

  if ( !buffer[0] ) {
    buffer++;
    len--;
    lenOffset = 1;
  }

  if ( wheelState.endpointOut > 0 ) {
    if (( res = libusb_interrupt_transfer ( wheelState.usbHandle,
        wheelState.endpointOut, buffer, len, &xferred, 1000 )) < 0 ) {
      return -1;
    }
  } else {
    if (( res = libusb_control_transfer ( wheelState.usbHandle,
        LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_INTERFACE |
        LIBUSB_ENDPOINT_OUT, 0x09, ( 2 << 8 ) | reportNum,
        wheelState.interfaceNum, buffer, len, 1000 )) < 0 ) {
      return -1;
    }
    xferred = len;
  }
  xferred += lenOffset;
  return xferred;
}


int
_sxWheelRead ( oaFilterWheel* wheel, char* buffer, int maxlen )
{
  int xferred;
  int res;

  buffer[0] = 0;
  buffer[1] = 0;
  if (( res = libusb_interrupt_transfer ( wheelState.usbHandle,
      wheelState.endpointIn, buffer, maxlen, &xferred, 5000 )) < 0 ) {
    return -1;
  }

  if ( xferred != 2 ) {
    return -1;
  }

  return xferred;
}
